#include <sys/types.h>
#include <sys/socket.h>
#include <stdlib.h>
#include <strings.h>
#include <netdb.h>
#include <stdio.h>
#include "sequdp.h"




struct header {
	int seqno;
	size_t len;	/* size of data */
};

	
struct seqmessage {
	struct header hdr;
	char data[MAXMSGSIZE];
};

typedef struct seqmessage *seqmessage_t;

static int seqno;

int
seqinit()
{
	seqno = 0;
}


static seqmessage_t
alloc(size_t len)
{
	return (seqmessage_t)malloc(sizeof(struct header) + len);
}



int
seqsendto(int s, const void *msg, size_t len, int flags,
	  const struct sockaddr *to, socklen_t tolen)
{
	int cc;
	seqmessage_t m;

	if (len > MAXMSGSIZE) {
		fprintf(stderr, "seqsendto: len too big");
		return -1;
	}

	
	m = alloc(len);
	if (m == 0) {
		fprintf(stderr,"seqsendto:out of memory\n");
		return -1;
	}
	m->hdr.seqno = seqno++;
	m->hdr.len = len;
	bcopy(msg, &(m->data[0]), len);
	cc = sendto(s, m, len + sizeof(m->hdr), flags, to, tolen);
	free (m);
	return cc;
}


	

int
seqrecvfrom(int s, void *buf, size_t len, int flags, struct sockaddr *from, socklen_t *fromlen)
{
	int cc;
	seqmessage_t m;
	size_t msglen;
	size_t datalen;

	m = alloc(MAXMSGSIZE);
	msglen = sizeof(struct seqmessage);
	
	cc = recvfrom(s, m, msglen, flags, from, fromlen);

	if (cc <= 0)
		return cc;

	showSenderInfo(from, *fromlen);
	
	datalen = cc - sizeof(m->hdr);
	
	if (cc < sizeof(m->hdr) || m->hdr.len != datalen) {
		fprintf(stderr,"Bad header\n");
		return -1;
	}

	fprintf(stderr, "Received seq number %d bytes %d\n", m->hdr.seqno, m->hdr.len);		
	if (datalen > len)
		datalen = len;
	bcopy(m->data, buf, datalen);
	return datalen;
}


void
showSenderInfo(struct sockaddr *sa, socklen_t fromlen)
{
	char hostname[NI_MAXHOST];
	char portnumber[NI_MAXSERV];
	
	if (getnameinfo( sa, fromlen,
			 hostname, sizeof(hostname),
			 portnumber, sizeof(portnumber),
			 NI_NUMERICSERV) < 0)
		error("awaitRequest: getnameinfo");
	fprintf(stderr,"Message from %s.%s\n", hostname, portnumber);
}

